import datetime

from django.contrib.auth import get_user_model
from factory import DjangoModelFactory, Sequence, \
    SubFactory, fuzzy, PostGenerationMethodCall, post_generation, compat

from arctic.models import Role, UserRole
from articles.models import Article, Category, Tag


class CategoryFactory(DjangoModelFactory):
    name = Sequence(lambda n: 'Category {}'.format(n))

    class Meta:
        model = Category


class TagFactory(DjangoModelFactory):
    term = Sequence(lambda n: 'Tag {}'.format(n))

    class Meta:
        model = Tag


class ArticleFactory(DjangoModelFactory):
    title = Sequence(lambda n: 'Article {}'.format(n))
    category = SubFactory(CategoryFactory)
    published = fuzzy.FuzzyChoice([True, False])
    updated_at = fuzzy.FuzzyDateTime(datetime.datetime(2016, 1, 1,
                                                       tzinfo=compat.UTC))

    class Meta:
        model = Article


class UserFactory(DjangoModelFactory):
    email = Sequence(lambda n: 'user{}@test.com'.format(n))
    username = Sequence(lambda n: 'user{}'.format(n))
    password = PostGenerationMethodCall('set_password', 'password')
    is_active = True

    @post_generation
    def set_urole(self, create, *args, **kwargs):
        if create:
            UserRole.objects.create(
                user=self, role=Role.objects.get(name='editor'))

    class Meta:
        model = get_user_model()



import pytest

from django.urls import reverse


@pytest.mark.django_db
class TestDataListView(object):
    """
    Test ListView using ArticleListView from example application
    """
    def _request(self, admin_client):
        response = admin_client.get(reverse('countries-list'))
        assert response.status_code == 200
        return response

    def _assert_list_items_len(self, response, length):
        assert 'list_items' in response.context_data
        assert len(response.context_data['list_items']) == length

    # def test_paginated_items(self, admin_client):
    #     """
    #     Paginated List view
    #     """
    #     response = self._request(admin_client)
    #     self._assert_list_items_len(response, 250)

    #     assert response.context_data['page_obj'].paginator.num_pages == 20

    # def test_simple_search_form(self, admin_client):
    #     # empty search, return all
    #     response = admin_client.get(reverse('countries-list'),
    #                                 {'search': ''})
    #     assert len(response.context_data['list_items']) == 150

    #     # search filled, filter content
    #     response = admin_client.get(reverse('countries-list'),
    #                                 {'search': 'af'})
    #     assert len(response.context_data['list_items']) == 3



import pytest
from django.contrib.auth import get_user_model

from tests.factories import ArticleFactory, TagFactory

User = get_user_model()


class FormCustomMock(object):
    """
    Custom mock to mimic the get_form() method for the Create and
    Update View. It needs to support self.get_form().fields and
    self.get_form()['fieldname']
    """
    fields = None

    def __init__(self, form):
        self.fields = {field.name: field for field in form}

    def __getitem__(self, key):
        return self.fields[key]

    def get_form(self):
        return self


def get_form(form):
    """
    Helper function to make sure you don't need to know how the
    FormCustomMock works

    Call this one, when you want to have the get_form-method
    """
    form_mock = FormCustomMock(form)
    return form_mock.get_form


@pytest.fixture
def tag():
    return TagFactory()


@pytest.fixture
def article(tag):
    article = ArticleFactory()
    article.tags.add(tag)
    return article



from django.core.paginator import Paginator, Page, EmptyPage
from django.utils.functional import cached_property


class IndefinitePaginator(Paginator):
    has_next = False

    @cached_property
    def count(self):
        try:
            return super(IndefinitePaginator, self).count
        except ValueError:
            # count is unknown, but we still want pagination
            return -1

    @cached_property
    def num_pages(self):
        if self.count == -1:
            return -1
        return super(IndefinitePaginator, self).num_pages

    def page(self, number):
        number = self.validate_number(number)
        bottom = (number - 1) * self.per_page
        top = bottom + self.per_page
        # we don't know the total amount of items, so we ask for one extra item
        # if it exists then there's a next page
        object_list = self.object_list[bottom : top + 1]
        if len(object_list) > self.per_page:
            self.has_next = True
            object_list = object_list[:-1]
        return self._get_page(object_list, number, self)

    def _get_page(self, *args, **kwargs):
        return IndefinitePage(*args, **kwargs)

    def validate_number(self, number):
        try:
            return super(IndefinitePaginator, self).validate_number(number)
        except EmptyPage:
            if self.count == -1:
                return number
            raise


class IndefinitePage(Page):
    def __init__(self, object_list, number, paginator):
        self.object_list = object_list
        self.number = number
        self.paginator = paginator

    def has_next(self):
        return self.paginator.has_next

    def end_index(self):
        return self.start_index() + len(self.object_list) - 1



import pytest

from arctic.models import Role
from django.urls import reverse

from tests.factories import UserFactory


@pytest.mark.django_db
class TestUserCreate(object):
    @property
    def url(self):
        return reverse('users:create')

    def test_create_page(self, admin_client):
        response = admin_client.get(self.url)
        assert response.status_code == 200

    def test_user_submit(self, django_user_model, admin_client):
        assert django_user_model.objects.count() == 1

        username = 'u1'
        email = 'u1@test.test'

        response = admin_client.post(
            self.url, {'user-username': username,
                       'user-email': email,
                       'user-password1': 'qqqqqq22222',
                       'user-password2': 'qqqqqq22222',
                       'user-is_active': True,
                       'role-role': Role.objects.all()[0].pk}
        )
        assert response.status_code == 302
        assert django_user_model.objects.count() == 2
        user = django_user_model.objects.filter(
            username=username, email=email).first()
        assert user is not None

    def test_user_exist_submit(self, django_user_model, admin_client):
        username = 'u1'
        email = 'u1@test.test'
        UserFactory(username=username, email=email)
        assert django_user_model.objects.count() == 2

        response = admin_client.post(
            self.url, {'user-username': username,
                       'user-email': email,
                       'user-password1': 'qqqqqq22222',
                       'user-password2': 'qqqqqq22222',
                       'user-is_active': True,
                       'role-role': Role.objects.all()[0].pk}
        )
        assert response.status_code == 200
        assert django_user_model.objects.count() == 2



import pytest
from django.urls import reverse

from articles.views import ArticleUpdateView
from tests.factories import ArticleFactory


@pytest.mark.django_db
class TestArticleUpdate(object):

    def get_url(self, pk):
        return reverse('articles:detail', kwargs={'pk': pk})

    def test_update_page(self, admin_client):
        article = ArticleFactory()

        response = admin_client.get(self.get_url(article.pk))
        assert response.status_code == 200

    def test_readonly_fields(self, admin_client):
        article = ArticleFactory()
        view = ArticleUpdateView
        # Check with readonly field enabled
        view.readonly_fields = ['title']
        response = admin_client.get(self.get_url(article.pk))
        assert response.context_data['form'].fields['title']\
            .widget.attrs.get('readonly')

        # Check with readonly field disabled
        view.readonly_fields = []
        response = admin_client.get(self.get_url(article.pk))
        assert not response.context_data['form'].fields['title']\
            .widget.attrs.get('readonly')



# -*- coding: utf-8 -*-
# Generated by Django 1.9.6 on 2016-07-08 13:44
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Article',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('created_at', models.DateTimeField(auto_now=True)),
                ('updated_at', models.DateTimeField()),
                ('title', models.CharField(max_length=255, verbose_name='Title')),
                ('description', models.TextField(blank=True, verbose_name='Description')),
                ('published', models.BooleanField(default=False, verbose_name='Published')),
            ],
        ),
        migrations.CreateModel(
            name='Category',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, unique=True, verbose_name='Name')),
            ],
        ),
        migrations.CreateModel(
            name='Tag',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('term', models.CharField(max_length=255, unique=True, verbose_name='Term')),
            ],
        ),
        migrations.AddField(
            model_name='article',
            name='category',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='articles.Category', verbose_name='Category'),
        ),
        migrations.AddField(
            model_name='article',
            name='tags',
            field=models.ManyToManyField(to='articles.Tag'),
        ),
    ]



import importlib
import json

from django.conf import settings
from django.core.exceptions import PermissionDenied
from django.http import HttpResponse, JsonResponse
from django.utils.decorators import method_decorator
from django.views.decorators.csrf import csrf_exempt
from django.views.generic.base import TemplateView as BaseTemplateView
from django.views.generic.base import View as BaseView

from arctic.generics import TemplateView


class BadRequestView(TemplateView):
    page_title = "Bad Request"
    template_name = "arctic/400.html"
    permission_required = ""

    def get(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context, status=400)


class ForbiddenView(TemplateView):
    page_title = "Access Forbidden"
    template_name = "arctic/403.html"
    permission_required = ""

    def get(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context, status=403)


class NotFoundView(TemplateView):
    page_title = "Page not Found"
    template_name = "arctic/404.html"
    permission_required = ""

    def get(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context, status=404)


class InternalErrorView(TemplateView):
    page_title = "Internal Error"
    template_name = "arctic/500.html"
    permission_required = ""

    def get(self, request, *args, **kwargs):
        context = self.get_context_data(**kwargs)
        return self.render_to_response(context, status=500)


class AutoCompleteView(BaseView):
    max_items = 10

    def get_data(self, model, field, search):
        field_startswith = {field + "__istartswith": search}
        max_items = self.max_items
        data = []
        if search:
            for row in model.objects.filter(**field_startswith)[:max_items]:
                data.append({"label": getattr(row, field), "value": row.pk})
        return {"options": data}

    def get(self, request, *args, **kwargs):
        if settings.DEBUG or request.is_ajax():
            model_name, field = settings.ARCTIC_AUTOCOMPLETE[
                kwargs["resource"]
            ]
            model_path = ".".join(model_name.split(".")[:-1]) + ".models"
            model_package = importlib.import_module(model_path)
            model = getattr(model_package, model_name.split(".")[-1])
            context = self.get_data(model, field, kwargs["search"])
            return self.render_to_response(context)
        raise PermissionDenied

    def render_to_response(self, context, **response_kwargs):
        return JsonResponse(context, safe=False)


class OrderView(BaseView):
    http_method_names = ["post"]

    @method_decorator(csrf_exempt)
    def dispatch(self, request, *args, **kwargs):
        return super(OrderView, self).dispatch(request, *args, **kwargs)

    def post(self, request, resource):
        if settings.DEBUG or request.is_ajax():
            m, v = resource.rsplit(".", 1)
            module = importlib.import_module(m)
            view = getattr(module, v)
            view.reorder(json.loads(request.body))
            return HttpResponse(status=201)
        raise PermissionDenied


class RedirectToParentView(BaseTemplateView):
    template_name = "arctic/redirect_to_parent.html"


handler400 = BadRequestView.as_view()
handler403 = ForbiddenView.as_view()
handler404 = NotFoundView.as_view()
handler500 = InternalErrorView.as_view()



import pytest
from django.urls import reverse
from tests.factories import ArticleFactory


@pytest.mark.django_db
class TestPagination(object):
    """
    Test Arctic pagination
    """
    def test_paginated_list(self, admin_client):
        [ArticleFactory() for i in range(20)]
        response = admin_client.get("%s?page=2" % reverse('articles:list'))
        assert response.status_code == 200

    def test_page_out_of_list(self, admin_client):
        [ArticleFactory() for i in range(20)]
        response = admin_client.get("%s?page=20" % reverse('articles:list'))
        assert response.status_code == 404



import pytest
from django.core.exceptions import ImproperlyConfigured
from django.urls import reverse


@pytest.mark.django_db
class TestEmptyLogin(object):
    def test_empty_login_and_logout_with_no_required_login(
            self, client, settings
    ):
        settings.LOGIN_URL = None
        settings.LOGOUT_URL = None
        response = client.get(reverse('index'))
        assert response.status_code == 200

    def test_empty_login_and_logout_with_required_login(
            self, client, settings
    ):
        settings.LOGIN_URL = None
        settings.LOGOUT_URL = None

        with pytest.raises(ImproperlyConfigured):
            client.get(reverse('articles:list'))



import pytest

from django.urls import reverse

from tests.factories import UserFactory


@pytest.mark.django_db
class TestUserList(object):

    @property
    def url(self):
        return reverse('users:list')

    def test_user_list(self, admin_client):
        [UserFactory() for i in range(10)]
        response = admin_client.get(self.url)
        assert response.status_code == 200



# inspired and partially copied from
# https://github.com/jmcclell/django-bootstrap-pagination

import re

import django

from django.template import (
    Node,
    Library,
    TemplateSyntaxError,
    VariableDoesNotExist,
)
from django.template.loader import get_template

# As of django 1.10, template rendering no longer accepts a context, but
# instead accepts only accepts a dict. Up until django 1.8, a context was
# actually required. Fortunately Context takes a single dict parameter,
# so for django >=1.9 we can get away with just passing a unit function.
from arctic.utils import arctic_setting

if django.VERSION < (1, 9, 0):
    from django.template import Context
else:

    def Context(x):
        return x


register = Library()


def get_current_app(context):
    try:
        current_app = context.request.current_app
    except AttributeError:
        try:
            current_app = context.request.resolver_match.namespace
        except AttributeError:
            return None
    return current_app


def str_to_bool(val):
    """
    Helper function to turn a string representation of "true" into
    boolean True.
    """
    if isinstance(val, str):
        val = val.lower()

    return val in ["true", "on", "yes", True]


class PaginationNode(Node):
    """
    Render the pagination bar with the given parameters
    """

    def __init__(self, page, kwargs):
        self.page = page
        self.kwargs = kwargs

    def render(self, context):
        kwargs = {}

        # Retrieve variable instances from context where necessary
        for argname, argvalue in self.kwargs.items():
            try:
                kwargs[argname] = argvalue.resolve(context)
            except AttributeError:
                kwargs[argname] = argvalue
            except VariableDoesNotExist:
                kwargs[argname] = None

        # Unpack our keyword arguments, substituting defaults where necessary

        PAGINATION_SETTINGS = arctic_setting("ARCTIC_PAGINATION")

        show_label = str_to_bool(
            kwargs.get("show_label", PAGINATION_SETTINGS["show_label"])
        )
        show_first_last = str_to_bool(
            kwargs.get(
                "show_first_last", PAGINATION_SETTINGS["show_first_last"]
            )
        )
        range_length = kwargs.get("range", PAGINATION_SETTINGS["range"])
        if range_length is not None:
            range_length = int(range_length)

        # Generage our viewable page range
        page = self.page.resolve(context)
        paginator = kwargs.pop("paginator", None)

        page_count = paginator.num_pages if paginator else 1
        current_page = page.number if page else 1

        if page_count == -1:
            # assume total objects length is unknown, pages are not required
            show_first_last = False
            page_range = None
            first_page_index = None
            last_page_index = None
        else:
            if range_length is None:
                range_min = 1
                range_max = page_count
            else:
                if range_length < 1:
                    raise Exception(
                        'Optional argument "range" '
                        "expecting integer greater than 0"
                    )
                elif range_length > page_count:
                    range_length = page_count

                range_length -= 1
                range_min = max(current_page - (range_length // 2), 1)
                range_max = min(current_page + (range_length // 2), page_count)
                range_diff = range_max - range_min
                if range_diff < range_length:
                    shift = range_length - range_diff
                    if range_min - shift > 0:
                        range_min -= shift
                    else:
                        range_max += shift

            page_range = range(range_min, range_max + 1)

            first_page_index = 1
            last_page_index = page_count

        return get_template(
            arctic_setting("ARCTIC_PAGINATION_TEMPLATE")
        ).render(
            Context(
                {
                    "page_obj": page,
                    "paginator": paginator,
                    "request": context["request"],
                    "show_first_last": show_first_last,
                    "page_range": page_range,
                    "first_page_index": first_page_index,
                    "last_page_index": last_page_index,
                    "show_label": show_label,
                }
            )
        )


@register.tag
def arctic_paginate(parser, token):
    """
    Renders a Page object with pagination bar.
    Example::
        {% arctic_paginate page_obj paginator=page_obj.paginator range=10 %}
    Named Parameters::
        range - The size of the pagination bar (ie, if set to 10 then, at most,
                10 page numbers will display at any given time) Defaults to
                None, which shows all pages.
        show_first_last - Accepts "true" or "false". Determines whether or not
                          to show the first and last page links. Defaults to
                          "false"
    """
    bits = token.split_contents()
    if len(bits) < 2:
        raise TemplateSyntaxError(
            "'%s' takes at least one argument"
            " (Page object reference)" % bits[0]
        )
    page = parser.compile_filter(bits[1])
    kwargs = {}
    bits = bits[2:]

    kwarg_re = re.compile(r"(\w+)=(.+)")

    if len(bits):
        for bit in bits:
            match = kwarg_re.match(bit)
            if not match:
                raise TemplateSyntaxError(
                    "Malformed arguments to bootstrap_pagination paginate tag"
                )
            name, value = match.groups()
            kwargs[name] = parser.compile_filter(value)

    return PaginationNode(page, kwargs)



from django.contrib.auth import get_user_model
from django.urls import reverse_lazy
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from arctic.generics import CreateView, ListView, UpdateView
from arctic.loading import get_user_role_model

from .forms import UserChangeMultiForm, UserCreationMultiForm

User = get_user_model()
UserRole = get_user_role_model()
username_field = User.USERNAME_FIELD


class UserListView(ListView):
    page_title = _("Users")
    paginate_by = 20
    permission_required = "view_user"
    model = UserRole
    fields = [
        ("user__{}".format(username_field), _("Username")),
        ("role__name", _("Role")),
        ("user__is_active", _("Active")),
        ("user__last_login", _("Last Login")),
    ]

    ordering_fields = [
        "user__{}".format(username_field),
        "role__name",
        "user__last_login",
    ]

    search_fields = ["user__{}".format(username_field)]

    field_links = {"user__{}".format(username_field): "users:detail"}
    tool_links = [(_("Add User"), "users:create", "fa-plus")]

    def get_user__is_active_field(self, row_instance):
        symbol = "fa-check" if row_instance.user.is_active else "fa-minus"
        return mark_safe('<i class="fa {}"></i>'.format(symbol))


class UserCreateView(CreateView):
    page_title = _("Add User")
    model = UserRole
    success_url = reverse_lazy("users:list")
    form_class = UserCreationMultiForm
    permission_required = "add_user"

    def get_success_message(self, cleaned_data):
        return _("User {} was successfully created").format(
            self.object["user"]
        )


class UserUpdateView(UpdateView):
    page_title = _("Change User")
    model = UserRole
    success_url = reverse_lazy("users:list")
    form_class = UserChangeMultiForm
    permission_required = "change_user"

    def get_form_kwargs(self):
        kwargs = super(UserUpdateView, self).get_form_kwargs()
        kwargs.update(instance={"user": self.object.user, "role": self.object})
        return kwargs

    def get_success_message(self, cleaned_data):
        return _("User {} was successfully updated").format(
            self.object["user"]
        )



from django.conf.urls import url

from arctic.views import AutoCompleteView, OrderView, RedirectToParentView

app_name = "arctic"

urlpatterns = [
    url(
        r"^autocomplete/(?P<resource>[\w_]+)/(?P<search>[^/]*)$",
        AutoCompleteView.as_view(),
        name="autocomplete",
    ),
    url(r"^order/(?P<resource>[\w_\.]+)$", OrderView.as_view(), name="order"),
    url(
        r"^redirect-to-parent$",
        RedirectToParentView.as_view(),
        name="redirect_to_parent",
    ),
]



import pytest

from django.urls import reverse

from articles.views import ArticleListView
from tests.factories import ArticleFactory


@pytest.mark.django_db
class TestVirtualFields(object):
    def _request(self, admin_client):
        response = admin_client.get(reverse('articles:list'))
        assert response.status_code == 200
        return response

    def _assert_list_items_len(self, response, length):
        assert 'list_items' in response.context_data
        assert len(response.context_data['list_items']) == length

    def test_virtual_field(self, admin_client):
        """
        Virtual field displayed in ListView
        """
        article = ArticleFactory()
        view = ArticleListView()
        view.fields = ['title', 'description', 'published', 'category']
        response = self._request(admin_client)
        self._assert_list_items_len(response, 1)

        item = response.context_data['list_items'][0]
        assert item['fields'][0]['value'] == article.title
        assert item['fields'][1]['value'] == article.description
        assert item['fields'][3]['value'] == article.category.name

    def test_missing_virtual_field(self, admin_client):
        """
        Error happens on wrong virtual field name
        """
        article = ArticleFactory()  # noqa
        view = ArticleListView()
        view.fields = ['title', 'description', 'published', 'virtual_field']
        response = self._request(admin_client)

        search_virtual_field = False
        for field in response.context_data['list_items'][0]['fields']:
            if field['type'] == 'field' and 'virtual_field' in field['field']:
                search_virtual_field = True

        assert search_virtual_field is False

    def test_missing_virtual_field_error(self, admin_client):
        """
        Error happens on wrong virtual field name
        """
        article = ArticleFactory()
        view = ArticleListView()
        view.fields = ['title', 'description', 'published', 'virtual_field']

        exc = pytest.raises(
            AttributeError, view.get_field_value, 'virtual_field', article)

        assert str(exc.value) == \
            "'Article' object has no attribute 'virtual_field'", \
            exc.value.message

    def test_missing_virtual_field_execution_attribute_error(self,
                                                             admin_client):
        """
        Error happens on wrong virtual field name
        """
        article = ArticleFactory()
        view = ArticleListView()
        view.fields = ['title', 'description', 'published', 'broken']
        view.get_broken_field = lambda obj: obj.unknown_field
        exc = pytest.raises(
            AttributeError, view.get_field_value, 'broken', article)
        assert str(exc.value) == \
            "'Article' object has no attribute 'unknown_field'", \
            exc.value.message



from __future__ import absolute_import, unicode_literals

from django.urls import reverse, reverse_lazy
from django.utils.safestring import mark_safe
from django.utils.translation import ugettext_lazy as _

from arctic.generics import (
    CreateView,
    DeleteView,
    ListView,
    TemplateView,
    UpdateView,
)
from arctic.generics import collapsible_gettext as _c
from collections import OrderedDict

from .forms import ArticleForm, AdvancedArticleSearchForm, FiltersAndSearchForm
from .inlines import ImagesInline
from .models import Article, Category, Tag


class DashboardView(TemplateView):
    template_name = "arctic/index.html"
    page_title = "Dashboard"
    permission_required = "view_dashboard"

    def get_context_data(self, **kwargs):
        context = super(DashboardView, self).get_context_data(**kwargs)
        return context


class ArticleListView(ListView):
    paginate_by = 10
    model = Article
    fields = ["title", "description", "published", "category", "tags"]
    ordering_fields = ["title", "description", "category", "published"]
    search_fields = ["title"]
    simple_search_form_class = FiltersAndSearchForm
    advanced_search_form_class = AdvancedArticleSearchForm
    breadcrumbs = (("Home", "index"), ("Article List", None))
    action_links = [
        ("detail", "articles:detail", "fa-edit"),
        ("delete", "articles:delete", "fa-trash"),
    ]
    field_links = {
        "title": "articles:detail",
        "category": ("articles:category-detail", "category_id"),
    }
    tool_links_collapse = 2
    tool_links = [(_("Create Article"), "articles:create", "fa-plus")]
    field_classes = {"published": ""}

    permission_required = "view_article"

    def get_field_actions(self, obj):
        actions = list(self.action_links)
        if obj.published:
            actions.pop(1)  # delete
        return actions

    def get_category_field(self, row_instance):
        return row_instance.category.name

    def get_published_field(self, row_instance):
        symbol = "fa-check" if row_instance.published else "fa-minus"
        return mark_safe('<i class="fa {}"></i>'.format(symbol))

    def get_category_ordering_field(self):
        return "category__name"

    def get_published_field_classes(self, row_instance):
        return "online" if row_instance.published else "offline"


class ArticleUpdateView(UpdateView):
    page_title = _("Edit Article")
    permission_required = "change_article"
    model = Article
    # success_url = reverse_lazy('articles:list')
    inlines = [ImagesInline]
    form_class = ArticleForm
    actions = [(_("Cancel"), "cancel"), (_("Save"), "submit")]
    layout = OrderedDict(
        [
            (_c("Basic Details"), ["title", ["category|4", "tags"]]),
            (
                _c("Body|Extra Information for this fieldset", True),
                ["description"],
            ),
            (_("Extended Details"), [["published|4", "updated_at"]]),
        ]
    )

    def get_success_url(self):
        return reverse("articles:detail", args=(self.object.pk,))


class ArticleCreateView(CreateView):
    page_title = _("Create Article")
    # fields = ['title', 'description', 'tags', 'category', 'published']
    model = Article
    inlines = [ImagesInline]
    form_class = ArticleForm
    permission_required = "add_article"
    layout = OrderedDict(
        [
            (_c("Basic Details"), ["title", ["category|4", "tags"]]),
            (
                _c("Body|Extra Information for this fieldset", True),
                ["description"],
            ),
            (_("Extended Details"), [["published|4", "updated_at"]]),
        ]
    )

    def get_success_url(self):
        return reverse("articles:detail", args=(self.object.pk,))


class ArticleDeleteView(DeleteView):
    model = Article
    # success_url = reverse_lazy('articles:list')
    permission_required = "delete_article"


class CategoryListView(ListView):
    page_title = _("Categories")
    model = Category
    fields = ["name"]
    field_links = {"name": "articles:category-detail"}
    tool_links = [(_("Create Category"), "articles:category-create")]
    modal_links = {
        "articles:category-detail": {"type": "iframe", "height": 377},
        "articles:category-create": {"type": "iframe", "height": 256},
    }
    permission_required = "view_category"
    sorting_field = "order"
    action_links = [("delete", "articles:category-delete", "fa-trash")]


class CategoryArticlesListView(ArticleListView):
    def dispatch(self, request, *args, **kwargs):
        self.pk = kwargs.pop("pk")
        return super(CategoryArticlesListView, self).dispatch(
            request, *args, **kwargs
        )

    # disable some settings from the default article list
    tool_links = []
    page_title = "Edit Category: Articles"
    breadcrumbs = None

    tabs = [
        ("Detail", ("articles:category-detail", "pk")),
        ("Related Articles", ("articles:category-articles-list", "pk")),
    ]
    permission_required = "view_category"

    def get_queryset(self):
        qs = super(CategoryArticlesListView, self).get_queryset()
        return qs.filter(category_id=self.pk)


class CategoryUpdateView(UpdateView):
    page_title = _("Edit Category")
    model = Category
    fields = "__all__"
    # success_url = reverse_lazy('articles:category-list')
    tabs = [
        ("Detail", ("articles:category-detail", "pk")),
        ("Related Articles", ("articles:category-articles-list", "pk")),
    ]
    actions = [
        (_("Delete"), ("articles:category-delete", "pk")),
        (_("Cancel"), "cancel"),
        (_("Save"), "submit"),
    ]
    permission_required = "change_category"


class CategoryCreateView(CreateView):
    page_title = _("Create Category")
    model = Category
    fields = ["name"]
    permission_required = "add_category"

    def get_success_url(self):
        if self.request.GET.get("inmodal"):
            return reverse("arctic:redirect_to_parent")
        return self.in_modal(str(self.success_url))  # success_url may be lazy


class CategoryDeleteView(DeleteView):
    model = Category
    redirect = True
    success_url = reverse_lazy("articles:category-list")
    permission_required = "delete_category"


class TagListView(ListView):
    page_title = _("Tags")
    model = Tag
    fields = ["term"]
    field_links = {"term": "articles:tag-detail"}
    tool_links = [(_("Create Tag"), "articles:tag-create")]
    permission_required = "view_tag"


class TagUpdateView(UpdateView):
    page_title = _("Edit Tag")
    model = Tag
    fields = "__all__"
    success_url = reverse_lazy("articles:tag-list")
    permission_required = "change_tag"


class TagCreateView(CreateView):
    page_title = _("Create Tag")
    model = Tag
    fields = ["term"]
    permission_required = "add_tag"

    def get_success_url(self):
        return reverse("articles:tag-detail", args=(self.object.pk,))


class TagDeleteView(DeleteView):
    model = Tag
    success_url = reverse_lazy("articles:tag-list")
    permission_required = "delete_tag"



import pytest

from django.forms import Form
from django.forms.widgets import Media

from arctic.generics import View, FormView


class FormWithAssets(Form):
    class Media:
        css = {
            'all': ('form-common.css', 'view-common.css')
        }
        js = ('form1-1.js', 'form-common.js')


class FormWithOutAssets(Form):
    pass


class AnotherFormWithAssets(Form):
    class Media:
        css = {
            'all': ('form-common.css', 'form2.css')
        }
        js = ('form2-1.js', 'form-common.js')


class ViewWithAssets(View):
    class Media:
        css = {
            'all': ('view.css', 'view-common.css')
        }
        js = ('view.js',)


class ViewWithForm(FormView, View):
    form_class = FormWithAssets


class FormViewWithOutFormAssets(FormView, View):
    form_class = FormWithOutAssets

    class Media:
        css = {
            'all': ('view.css', 'view-common.css')
        }
        js = ('view.js',)


class FormViewWithAssets(FormView, View):
    form_class = FormWithAssets

    class Media:
        css = {
            'all': ('view.css', 'view-common.css')
        }
        js = ('view.js',)


class FormViewWithAssetsAndExtraAssets(FormView, View):
    form_class = FormWithAssets

    class Media:
        css = {
            'all': ('view.css', 'view-common.css')
        }
        js = ('view.js',)

    def get_media_assets(self):
        return Media(js=['extra.js'])


class ViewWithAssetsAndForms(ViewWithAssets):
    form1 = FormWithAssets
    form2 = AnotherFormWithAssets


@pytest.mark.django_db
class TestViewAssets(object):
    """
    Test media assets in views, including eventual form assets.
    """

    def test_regular_view_with_assets(self):
        view = ViewWithAssets()
        response = """<link href="/static/view.css" type="text/css" media="all" rel="stylesheet">
<link href="/static/view-common.css" type="text/css" media="all" rel="stylesheet">
<script type="text/javascript" src="/static/view.js"></script>"""  # noqa

        assert str(view.media).replace(' />', '>') == response

    def _regular_view_with_media(self, settings):
        settings.ARCTIC_COMMON_MEDIA_ASSETS = {
            'css': {
                'all': ['common.css']
            }
        }
        view = ViewWithAssets()
        response = """<link href="/static/common.css" type="text/css" media="all" rel="stylesheet">
<link href="/static/view.css" type="text/css" media="all" rel="stylesheet">
<link href="/static/view-common.css" type="text/css" media="all" rel="stylesheet">
<script type="text/javascript" src="/static/view.js"></script>"""  # noqa

        assert str(view.media).replace(' />', '>') == response

    def test_view_with_assets_and_forms(self):
        view = ViewWithAssetsAndForms()
        response = """<link href="/static/view.css" type="text/css" media="all" rel="stylesheet">
<link href="/static/view-common.css" type="text/css" media="all" rel="stylesheet">
<script type="text/javascript" src="/static/view.js"></script>"""  # noqa

        assert str(view.media).replace(' />', '>') == response

    def test_form_view_with_form(self, rf):
        view = ViewWithForm()
        request = rf.get('/')
        view.request = request
        response = """<link href="/static/form-common.css" type="text/css" media="all" rel="stylesheet">
<link href="/static/view-common.css" type="text/css" media="all" rel="stylesheet">
<script type="text/javascript" src="/static/form1-1.js"></script>
<script type="text/javascript" src="/static/form-common.js"></script>"""  # noqa

        assert str(view.media).replace(' />', '>') == response

    def _form_view_with_form_and_common_assets(self, rf, settings):
        settings.ARCTIC_COMMON_MEDIA_ASSETS = {
            'js': ['common.js']
        }
        view = ViewWithForm()
        request = rf.get('/')
        view.request = request
        response = """<link href="/static/form-common.css" type="text/css" media="all" rel="stylesheet">
<link href="/static/view-common.css" type="text/css" media="all" rel="stylesheet">
<script type="text/javascript" src="/static/common.js"></script>
<script type="text/javascript" src="/static/form1-1.js"></script>
<script type="text/javascript" src="/static/form-common.js"></script>"""  # noqa

        assert str(view.media).replace(' />', '>') == response

    def _form_view_with_assets_and_form(self, rf):
        view = FormViewWithAssets()
        request = rf.get('/')
        view.request = request
        response = """<link href="/static/view.css" type="text/css" media="all" rel="stylesheet">
<link href="/static/view-common.css" type="text/css" media="all" rel="stylesheet">
<link href="/static/form-common.css" type="text/css" media="all" rel="stylesheet">
<script type="text/javascript" src="/static/view.js"></script>
<script type="text/javascript" src="/static/form1-1.js"></script>
<script type="text/javascript" src="/static/form-common.js"></script>"""  # noqa

        assert str(view.media).replace(' />', '>') == response

    def _form_view_with_assets_and_form_and_extra_assets(self, rf):
        view = FormViewWithAssetsAndExtraAssets()
        request = rf.get('/')
        view.request = request
        response = """<link href="/static/view.css" type="text/css" media="all" rel="stylesheet">
<link href="/static/view-common.css" type="text/css" media="all" rel="stylesheet">
<link href="/static/form-common.css" type="text/css" media="all" rel="stylesheet">
<script type="text/javascript" src="/static/view.js"></script>
<script type="text/javascript" src="/static/form1-1.js"></script>
<script type="text/javascript" src="/static/form-common.js"></script>
<script type="text/javascript" src="/static/extra.js"></script>"""  # noqa

        assert str(view.media).replace(' />', '>') == response

    def test_form_view_without_form_assets(self, rf):
        view = FormViewWithOutFormAssets()
        request = rf.get('/')
        view.request = request
        response = """<link href="/static/view.css" type="text/css" media="all" rel="stylesheet">
<link href="/static/view-common.css" type="text/css" media="all" rel="stylesheet">
<script type="text/javascript" src="/static/view.js"></script>"""  # noqa

        assert str(view.media).replace(' />', '>') == response

    def test_view_without_assets(self):
        view = View()

        assert str(view.media) == ''



import pytest

from django.urls import reverse
from articles.models import Category, Tag
# from tests.factories import CategoryFactory, TagFactory


@pytest.mark.django_db
class TestCreateView(object):
    """
    Test CreateView using ArticleCreateView, CategoryCreateView
    and TagCreateView from example application
    """
    @property
    def article_url(self):
        return reverse('articles:create')

    @property
    def category_url(self):
        return reverse('articles:category-create')

    @property
    def tag_url(self):
        return reverse('articles:tag-create')

    def test_create_article_page(self, admin_client):
        response = admin_client.get(self.article_url)
        assert response.status_code == 200

    def test_create_category_page(self, admin_client):
        response = admin_client.get(self.category_url)
        assert response.status_code == 200

    def test_create_tag_page(self, admin_client):
        response = admin_client.get(self.tag_url)
        assert response.status_code == 200

    # def test_article_submit(self, admin_client):
    #     CategoryFactory()
    #     TagFactory()
    #     cat_id = Category.objects.all()[0].pk
    #     tag_id = Tag.objects.all()[0].pk
    #     response = admin_client.post(
    #         self.article_url, {'title': 'Title_text',
    #                            'description': 'Description_text',
    #                            'published': True,
    #                            'category': cat_id,
    #                            'tags': tag_id,
    #                            'updated_at': '11-11-2016 10:53'}
    #     )

    #     articles_obj = Article.objects.count()
    #     assert response.status_code == 302
    #     assert articles_obj == 1

    def test_category_submit(self, admin_client):
        response = admin_client.post(
            self.category_url, {'name': 'test_category'}
        )
        category_obj = Category.objects.count()
        assert response.status_code == 302
        assert category_obj == 1

    def test_tag_submit(self, admin_client):
        response = admin_client.post(
            self.tag_url, {'term': 'test_tag'}
        )
        tag_obj = Tag.objects.count()
        assert response.status_code == 302
        assert tag_obj == 1



from __future__ import absolute_import, unicode_literals

import json
import os
from six.moves import urllib

from django.http import JsonResponse
from django.utils.safestring import mark_safe
from django.views.generic import TemplateView

from arctic.generics import DataListView
from arctic.utils import RemoteDataSet, offset_limit


class CountryAPIView(TemplateView):
    """
    Mock API used to demonstrate the usage of DataListView
    It's a simple implementation with support for sorting, pagination,
    filtering and field selection.
    """

    def render_to_response(self, context, **response_kwargs):
        data = []
        dir_path = os.path.dirname(os.path.realpath(__file__))
        with open(os.path.join(dir_path, "countries.json"), "r") as f:
            data = json.loads(f.read())

        if data:
            all_fields = data[0].keys()

        for field in all_fields:
            value = self.request.GET.get(field, None)
            if value:
                data = list(
                    filter(lambda d: value.lower() in d[field].lower(), data)
                )

        if self.request.GET.get("fields"):
            fields = self.request.GET.get("fields").split(",")
            fields = set(all_fields).intersection(fields)
            temp = []
            for row in data:
                temp_row = {}
                for field in fields:
                    temp_row[field] = row[field]
                temp.append(temp_row)
            data = temp

        if self.request.GET.get("order_by"):
            order_by = self.request.GET.get("order_by")
            reverse = False
            if order_by[0] == "-":
                reverse = True
                order_by = order_by[1:]

            data = sorted(data, key=lambda k: k[order_by], reverse=reverse)

        if self.request.GET.get("offset") or self.request.GET.get("limit"):
            offset = int(self.request.GET.get("offset", 0))
            limit = min(
                int(self.request.GET.get("limit", len(data) - offset)),
                len(data) - offset,
            )
            if offset > len(data) or (offset + limit) > len(data):
                data = []
            else:
                data = data[offset : limit + offset]

        return JsonResponse(data, safe=False, **response_kwargs)


class CountriesDataSet(RemoteDataSet):
    url_template = "countries-api/?{filters}{fields}{order}{paginate}"
    order_template = "&order_by={}"

    @offset_limit
    def get(self, offset, limit):
        r = urllib.request.urlopen((self.get_url(offset, limit)))
        data = r.read().decode("utf-8")
        return json.loads(data)


class CountryListView(DataListView):
    paginate_by = 10
    dataset = CountriesDataSet()
    fields = ["name", "capital", "flag"]
    ordering_fields = ["name", "capital"]
    search_fields = ["name"]
    breadcrumbs = (("Home", "index"), ("Country List", None))
    page_title = "Countries"
    permission_required = "country_view"

    def get_context_data(self, **kwargs):
        # small hack to setup the current host in the url of the local API
        if not self.dataset.url_template.startswith("http"):
            self.dataset.url_template = "{}://{}/{}".format(
                self.request.scheme,
                self.request.get_host(),
                self.dataset.url_template,
            )
        context = super(CountryListView, self).get_context_data(**kwargs)
        return context

    def get_flag_field(self, obj):
        return mark_safe(
            '<img style="max-width: 2rem" src="{}" />'.format(obj["flag"])
        )



from django.conf.urls import url

from .views import UserCreateView, UserListView, UserUpdateView

app_name = "users"

urlpatterns = [
    url(r"^$", UserListView.as_view(), name="list"),
    url(r"^create/$", UserCreateView.as_view(), name="create"),
    url(r"^(?P<pk>\d+)/$", UserUpdateView.as_view(), name="detail"),
]



from django.apps import AppConfig
from django.core.exceptions import ImproperlyConfigured


class ArcticConfig(AppConfig):
    name = "arctic"

    # even though db usage in AppConfig is not a recommended best practice
    # according to the Django docs, this is the best location to sync the
    # roles defined in settings.ARCTIC_ROLES to the database.
    def ready(self):
        from .mixins import RoleAuthentication

        try:
            RoleAuthentication.sync()
        except ImproperlyConfigured:
            raise
        except Exception:
            pass
        import arctic.signals  # noqa



# TODO: Move extra views into our own views. So we can override where needed
from extra_views import InlineFormSet

from .models import Article, Image


class ArticleInline(InlineFormSet):
    model = Article
    fields = "__all__"


class TagsInline(InlineFormSet):
    model = Article.tags.through
    fields = "__all__"


class ImagesInline(InlineFormSet):
    model = Image
    fields = "__all__"
    layout = {"blah": "bah"}



import os
import shutil
import sys
from setuptools import find_packages, setup

__VERSION__ = "1.3.5"


def read_md(f):
    try:
        from pypandoc import convert

        return convert(f, "rst")
    except ImportError:
        return open(f, "r").read()


def check_installed(*packages):
    exit = False
    # if not a list, let's make it into one
    for package in packages:
        if os.system("pip freeze | grep {} >/dev/null".format(package)):
            print("{0} not installed: use `pip install {0}`".format(package))
            exit = True
    if exit:
        print("Exiting.")
        sys.exit()


def check_tagged_version(version):
    current_branch_is_master = not os.system(
        "git rev-parse --abbrev-ref HEAD | grep master >/dev/null"
    )
    master_is_tagged = not os.system(
        "git tag --points-at master | grep {} >/dev/null".format(version)
    )
    if not current_branch_is_master:
        print("Publish can only be done from the master branch.")
    if not master_is_tagged:
        print(
            "`{}` tag has not been created on the master branch.".format(
                version
            )
        )
    if not (current_branch_is_master and master_is_tagged):
        print("Exiting.")
        sys.exit()


try:
    REQUIREMENTS = open("requirements/base.txt").read()
except Exception:
    REQUIREMENTS = None

if sys.argv[-1] == "publish":
    check_installed("pypandoc", "twine")
    check_tagged_version(__VERSION__)
    os.system("python setup.py sdist bdist_wheel")
    os.system("twine upload dist/*")
    shutil.rmtree("build")
    shutil.rmtree("dist")
    sys.exit()


setup(
    name="django-arctic",
    # See https://pypi.python.org/pypi?%3Aaction=list_classifiers
    classifiers=[
        "Development Status :: 5 - Production/Stable",
        "Environment :: Web Environment",
        "Intended Audience :: Developers",
        "Topic :: Software Development :: Libraries",
        "Topic :: Software Development :: Libraries :: Application Frameworks",
        "Topic :: Internet :: WWW/HTTP",
        "Framework :: Django",
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 2",
        "Programming Language :: Python :: 2.7",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.5",
        "Programming Language :: Python :: 3.6",
    ],
    version=__VERSION__,
    long_description=read_md("README.md"),
    install_requires=REQUIREMENTS,
    packages=find_packages(),
    include_package_data=True,
    entry_points={"console_scripts": ["arctic = arctic.bin.arctic:main"]},
    license="MIT",
    url="https://github.com/sanoma/django-arctic",
    author="Sanoma Netherlands",
    author_email="opensource@sanoma.com",
    description="Django framework to create custom content management systems",
)



import os
import shutil
import tempfile
from subprocess import call


def test_start_new_project():
    """
    Test creating a new project with project bootstrap command.
    """
    project_name = 'test_arctic_project'
    project_path = tempfile.mkdtemp()
    manage_py_bin = os.path.join(project_path, 'manage.py')

    # remove DJANGO_SETTINGS_MODULE form env or arctic command will try loading
    # 'example.settings' module.
    env = os.environ.copy()
    env.pop('DJANGO_SETTINGS_MODULE')

    args = ['arctic', 'start', project_name, project_path]
    retcode = call(args, env=env)

    assert retcode == 0
    assert os.path.exists(manage_py_bin)

    # run Django system checks of generated project
    args = [manage_py_bin, 'check']
    retcode = call(args, env=env)
    assert retcode == 0

    # run Django migrations of generated project
    args = [manage_py_bin, 'migrate']
    retcode = call(args, env=env)
    assert retcode == 0

    # cleanup test data
    shutil.rmtree(project_path)



"""
Django settings for example project.

Generated by 'django-admin startproject' using Django 1.9.4.

For more information on this file, see
https://docs.djangoproject.com/en/1.9/topics/settings/

For the full list of settings and their values, see
https://docs.djangoproject.com/en/1.9/ref/settings/
"""

import os

from .arctic import *

# Build paths inside the project like this: os.path.join(BASE_DIR, ...)
BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))

# Path helper
location = lambda x: os.path.join(
    os.path.dirname(os.path.realpath(__file__)), x
)

# Quick-start development settings - unsuitable for production
# See https://docs.djangoproject.com/en/1.9/howto/deployment/checklist/

# SECURITY WARNING: keep the secret key used in production secret!
SECRET_KEY = "(u$wqr4t^)-7)&3hc(o49-svh-em-y4$!u9kr99gavo(1pipdu"

# SECURITY WARNING: don't run with debug turned on in production!
DEBUG = True

ALLOWED_HOSTS = ["*"]

INTERNAL_IPS = ["127.0.0.1"]


# Application definition

INSTALLED_APPS = [
    "django.contrib.auth",
    "django.contrib.contenttypes",
    "django.contrib.sessions",
    "django.contrib.messages",
    "django.contrib.staticfiles",
    "django.contrib.humanize",
    "dashboard",
    "articles",
    "arctic",
    "arctic.users",
]


MIDDLEWARE = [
    "django.middleware.security.SecurityMiddleware",
    "django.contrib.sessions.middleware.SessionMiddleware",
    "django.middleware.common.CommonMiddleware",
    "django.middleware.csrf.CsrfViewMiddleware",
    "django.contrib.auth.middleware.AuthenticationMiddleware",
    "django.contrib.messages.middleware.MessageMiddleware",
    "django.middleware.clickjacking.XFrameOptionsMiddleware",
]

ROOT_URLCONF = "config.urls"

TEMPLATES = [
    {
        "BACKEND": "django.template.backends.django.DjangoTemplates",
        "DIRS": [os.path.join(BASE_DIR, "templates")],
        "APP_DIRS": True,
        "OPTIONS": {
            "context_processors": [
                "django.contrib.auth.context_processors.auth",
                "django.template.context_processors.debug",
                "django.template.context_processors.i18n",
                "django.template.context_processors.media",
                "django.template.context_processors.static",
                "django.template.context_processors.tz",
                "django.template.context_processors.request",
                "django.contrib.messages.context_processors.messages",
            ]
        },
    }
]

WSGI_APPLICATION = "config.wsgi.application"


# Database
# https://docs.djangoproject.com/en/1.9/ref/settings/#databases

DATABASES = {
    "default": {
        "ENGINE": "django.db.backends.sqlite3",
        "NAME": os.path.join(BASE_DIR, "db.sqlite3"),
    }
}

# Internationalization
# https://docs.djangoproject.com/en/1.9/topics/i18n/

USE_I18N = True

USE_L10N = True

LANGUAGE_CODE = "en"

TIME_ZONE = "Europe/Amsterdam"

USE_TZ = True


# Static files (CSS, JavaScript, Images)
# https://docs.djangoproject.com/en/1.9/howto/static-files/

STATIC_ROOT = location("static")
STATIC_URL = "/static/"

MEDIA_ROOT = location("media")
MEDIA_URL = "/media/"

LOGIN_URL = LOGOUT_URL = "login"


try:
    import debug_toolbar

    MIDDLEWARE.append("debug_toolbar.middleware.DebugToolbarMiddleware")
    INSTALLED_APPS.append("debug_toolbar")
except ImportError:
    pass



from __future__ import absolute_import, unicode_literals

from collections import OrderedDict

from django import template
from django.template import VariableDoesNotExist
from django.urls import reverse

register = template.Library()


# TODO Cleanup this file. Lot unnused


@register.simple_tag(takes_context=True)
def update_plugin_groups_counter(context):
    context["plugin_groups_counter"] += 1
    return ""


@register.filter()
def empty_plugin_form(formset, plugin_type):
    return formset.empty_plugin_form(plugin_type)


@register.filter()
def verbose_name(model):
    return model._meta.verbose_name


@register.filter()
def verbose_name_plural(model):
    return model._meta.verbose_name_plural


def get_parameters(parser, token):
    """
    {% get_parameters except_field %}
    """
    args = token.split_contents()
    if len(args) < 2:
        raise template.TemplateSyntaxError(
            "get_parameters tag takes at least 1 argument"
        )
    return GetParametersNode(args[1].strip())


class GetParametersNode(template.Node):
    """
    Renders current get parameters except for the specified parameter
    """

    def __init__(self, field):
        self.template_var = template.Variable(field)
        self.field = field

    def render(self, context):
        request = context["request"]
        getvars = request.GET.copy()

        # check if we pass template vat into tag
        try:
            self.field = self.template_var.resolve(context)
        except VariableDoesNotExist:
            pass

        if self.field in getvars:
            del getvars[self.field]

        if len(getvars.keys()) > 0:
            get_params = "%s&" % getvars.urlencode()
        else:
            get_params = ""

        return get_params


get_parameters = register.tag(get_parameters)


def get_all_fields(obj):
    """Returns a list of all field names on the instance."""
    fields = []
    for f in obj._meta.fields:
        fname = f.name

        get_choice = "get_" + fname + "_display"
        if hasattr(obj, get_choice):
            value = getattr(obj, get_choice)()
        else:
            try:
                value = getattr(obj, fname)
            except Exception:
                value = None

        if isinstance(value, list):
            value = ",".join(str(v) for v in value)

        if f.editable and value and f.name:
            fields.append(
                {"label": f.verbose_name, "name": f.name, "value": value}
            )

    return fields


@register.simple_tag(takes_context=True)
def query_string(context, **kwargs):
    """Add param to the given query string"""
    params = context["request"].GET.copy()

    for key, value in list(kwargs.items()):
        params[key] = value

    return "?" + params.urlencode()


@register.simple_tag(takes_context=True)
def query_string_ordering(context, value, **kwargs):
    """
    Add ordering param to the given query string
    :param context: template context
    :param value: examples would be '-id' or 'id'. A minus indicates that the
                  default sorting is descending
    :param kwargs: not used
    :return: Adjusted query string, starting with '?'
    """
    params = context["request"].GET.copy()

    # if the given value is '-id', it's core value would be 'id'
    core_value = value
    default_order = "asc"

    if core_value[0] == "-":
        core_value = value[1:]
        default_order = "desc"

    current_value = ""

    # by preference get the current ordering value from the filter
    # so that even if no explicit ordering is in the URL, we still
    # get the implicit ordering, the page's default
    # See generics.filters.FilterSet
    if "filter" in context:
        current_value = context["filter"].ordered_value()
    elif "ordering" in params:
        current_value = params["ordering"]

    # The first two clauses check if the current ordering is on the
    # same field as the desired one, in which case we reverse the direction.
    # If it's on another field, we use the default direction.
    if current_value == core_value:
        order_prefix = "-"
    elif current_value == "-" + core_value:
        order_prefix = ""
    elif default_order == "desc":
        order_prefix = "-"
    else:
        order_prefix = ""

    params["ordering"] = order_prefix + core_value
    return "?" + params.urlencode()


@register.simple_tag(takes_context=True)
def arctic_url(context, link, *args, **kwargs):
    """
    Resolves links into urls with optional
    arguments set in self.urls. please check get_urls method in View.

    We could tie this to check_url_access() to check for permissions,
    including object-level.
    """

    def reverse_mutable_url_args(url_args):
        mutated_url_args = []
        for arg in url_args:

            # listview item, and argument is a string
            if "item" in context and type(arg) == str:
                # try to get attribute of this object
                try:
                    arg = getattr(context["v"], arg.split(".")[-1])
                # if not found fallback to row pk, which is always first column
                except Exception:
                    arg = context["item"][0]
            mutated_url_args.append(arg)
        return reverse(link, args=mutated_url_args, kwargs=None)

    url_args = args

    # set arguments defined in urls if provided
    if type(link) in (tuple, list):
        context["urls"][link[0]] = list(link[1:])
        link = link[0]
    if link in context["urls"]:

        # for where the params directly given. e.g. ('article-detail',
        # (self.object.pk,))
        url_args = context["urls"][link]

        # list given, which means it's mutable!
        if isinstance(url_args, list):
            return reverse_mutable_url_args(url_args)

    return reverse(link, args=url_args, kwargs=None)


@register.filter()
def lookup(dct, key):
    return dct.get(key)


@register.filter()
def typename(obj):
    return type(obj).__name__


@register.filter
def index(lst, i):
    return lst[int(i)]


@register.filter
def keys(obj):
    if type(obj) in (dict, OrderedDict):
        return list(obj.keys())
    return []


@register.filter
def get_item(dictionary, key):
    return dictionary.get(key)


@register.filter
def get_attr(obj, item, default=None):
    return getattr(obj, item, default)


@register.filter
def replace(s, old, new):
    return s.replace(old, new)



# Generated by Django 2.0.6 on 2018-06-18 08:45

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('arctic', '0001_initial'),
    ]

    operations = [
        migrations.AlterModelOptions(
            name='role',
            options={'ordering': ['name']},
        ),
        migrations.AlterModelOptions(
            name='userrole',
            options={'ordering': ['user']},
        ),
    ]
